{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J-,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$MINSTACKSIZE $00004000}
{$MAXSTACKSIZE $00100000}
{$IMAGEBASE $00400000}
{$APPTYPE GUI}
unit Splasher;

{ Richard A. DeVenezia
  http://www.devenezia.com

  usage:
  Splash <image> <duration>

  <image> is
  filename - if wildcarded, each match bmp/jpg/png is shown; if a directory, all
             bmp/jpg/png files in folder are shown in alphabetical order
  off

  <duration> is
  > 0 number of seconds to show image before showing next image
      after last image is shown close thyself
  = 0 image shown forever
  < 0 number of seconds to show image before showing next image
      after last image is shown start over (loop)

  The next image depends on the current order, forward or reverse
  Single click - show next image
  Double click - show next image
  Right click - show window border
  Space - forward order, show next image
  Backspace - reverse order, show next image
  Esc, q, Q - quit

  Version 1.0
  12/02/03 Initial issue
  12/03/03 Add exception handling and duration 0 handling
           Add wildcard filenames and -duration handling
           Add usage dialog
  12/05/03 Add forward and reverse
           Add Esc, Q, and q handling
           Add right click to show border
           Add H and h to display usage dialog

  Version 1.1
  12/14/03 Add @filelist handling
           Add ? to display usage dialog

  Version 1.2
  08/31/09 Add png handling
           The Gustavo Daud source code was downloaded from
           http://www.isheu.com/download/fs0713/zip/2008/8/17/TPNGImage%20component%20for%20Delphi.zip
           (it can also be downloaded from
             http://www.torry.net/authorsmore.php?id=6929
           )
           Unzip the source to a folder of your choosing, and add that folder
           to the Search Path found under Project/Options:Directories
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms
, Dialogs, ExtCtrls, StrUtils
, Jpeg
, SingleInstance
, pngimage
  ;

type
    TForm1 = class(TForm)
    Image1: TImage;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure Image1Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Image1DblClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Image1ContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
  private
    { Private declarations }
    FileList: TStringList;
    Caption: String;
    WasBordered : Boolean;
    procedure ProcessNewCommandLine(var Msg: tMessage); message wmNewCommandLine;
    function  ProcessImageArg(Path:String;CanAlert:Boolean) : Boolean;
    procedure AddToFileList(Path:String);
    function  HandledType (FileName:String): Boolean;
    procedure LoadImageFile();
    procedure Alert(Text:String);
    procedure Usage();
  public
    { Public declarations }
  end;

const
  DefaultDuration : Cardinal = 8;

var
  Form1: TForm1;
  Duration : Integer;
  TimerTicks : Integer;
  FileIndex : Integer = -1;
  Increment : Integer = 1;
  Looping : Boolean = False;

implementation

{$R *.dfm}

procedure TForm1.Alert (Text: String);
begin
  Application.MessageBox(PChar(Text),PChar(Caption),MB_OK);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  Code : Integer;
begin
  FileList := TStringList.Create;
  WasBordered := False;
  Caption := ExtractFileName(ParamStr(0));

  Form1.DoubleBuffered := True;

  if (ParamCount = 0) or SameText ('-?', ParamStr(1)) or SameText('-help', ParamStr(1)) then begin
    Usage();
    Application.Terminate;
    Exit;
  end;

  if not ProcessImageArg (ParamStr(1), true) then Exit;

  if ParamCount > 1 then
    Val (ParamStr(2), Duration, Code)
  else
    Code := -1;

  if Code <> 0 then Duration := DefaultDuration;

  if Duration < 0 then begin
    Looping := True;
    Duration := -Duration;
  end;

  LoadImageFile ();
end;


procedure TForm1.FormDestroy(Sender: TObject);
begin
  FileList.Free;
end;


procedure TForm1.FormClick(Sender: TObject);
begin
  LoadImageFile();
end;


procedure TForm1.Image1Click(Sender: TObject);
begin
  LoadImageFile();
end;


procedure TForm1.Image1DblClick(Sender: TObject);
begin
  LoadImageFile();
end;


procedure TForm1.Timer1Timer(Sender: TObject);
begin
  Inc (TimerTicks);
  if TimerTicks * Integer(Timer1.Interval) >= Duration * 1000 then begin
    if Not Looping
    and (((Increment > 0) and (FileIndex >= FileList.Count-1))
         or
         ((Increment < 0) and (FileIndex <= 0))
        )
    then
      Application.Terminate
    else
      LoadImageFile();
  end;
end;


procedure TForm1.ProcessNewCommandLine(var Msg: tMessage);
var S: String;
    PC: array[0..MAX_PATH]of Char;
    Param : String;
begin
  GlobalGetAtomName(Msg.wParam, PC, MAX_PATH);
  S:= StrPas(PC);

  Param := Copy (S,0,Pos(#10,S)-1);
  S:= Copy (S,Pos(#10,S)+1, Length(S));

  if not ProcessImageArg(Param, false) then Exit;
  LoadImageFile();
end;


function TForm1.ProcessImageArg(Path:String; CanAlert:Boolean) :Boolean;
var
  FileName : String;
  F: TextFile;
begin;
  Timer1.Enabled := False;

  if SameText (Path, 'off') then begin
    Application.Terminate;
    ProcessImageArg := False;
    Exit;
  end;

  FileList.Clear;

  AddToFileList (Path);

  FileList.Sort();

  if (FileList.Count = 0) and (Copy(Path,1,1) = '@') then begin
    { @ prefaced argument can only list one file to look in }
    FileName := Copy (Path,2,Length(Path)-1);
    if FileExists (FileName) then begin
      AssignFile (F, FileName);
      FileMode := fmOpenRead;
      Reset (F);
      While not Eof(F) do begin
        ReadLn (F, FileName);
        if FileExists (FileName) then
          FileList.Append ( FileName );
      end;
    end;
  end;

  if (FileList.Count = 0) then begin
    if CanAlert then Alert (Path + ': no images.');
    Application.Terminate;
    ProcessImageArg := False;
    Exit;
  end;

  Timer1.Enabled := Duration <> 0;
  ProcessImageArg := True;
end;


procedure TForm1.AddToFileList(Path:String);
var
  SearchRec: TSearchRec;
  FilePath : String;
  rc : Integer;
begin
  if DirectoryExists (Path) then
    Path := ExcludeTrailingPathDelimiter(Path) + '\*.*';

  FilePath := ExtractFilePath(Path);

  rc := FindFirst (Path,  faAnyFile, SearchRec);
  while (rc = 0) do begin
    if HandledType (SearchRec.Name)

    then begin
      FileList.Append ( FilePath+SearchRec.Name );
    end;

    rc := FindNext (SearchRec);
  end;
  FindClose (SearchRec);
end;


function TForm1.HandledType (FileName:String): Boolean;
begin
  HandledType :=
       AnsiEndsText('.bmp',  FileName)
    or AnsiEndsText('.jpg',  FileName)
    or AnsiEndsText('.jpeg', FileName)
    or AnsiEndsText('.png', FileName)
  ;
end;


procedure TForm1.LoadImageFile();
  var FileName : String;
  var oldW, newW, oldH, newH : Integer;
  var centerX, centerY : Integer;
begin
  Timer1.Enabled := False;
  TimerTicks := 0;

  if FileIndex = -1 then
    FileIndex := 0
  else
    FileIndex := FileIndex + Increment;

  if FileIndex < 0 then begin
    if Not Looping then begin
      Application.Terminate;
      Exit;
    end;
    FileIndex := FileList.Count-1
  end
  else
  if FileIndex >= FileList.Count then begin
    if Not Looping then begin
      Application.Terminate;
      Exit;
    end;
    FileIndex := 0;
  end;

  FileName := FileList[FileIndex];
  try
    oldW := Image1.Width;
    oldH := Image1.Height;

    centerX := Form1.Left + Form1.Width div 2;
    centerY := Form1.Top  + Form1.Height div 2;

    Image1.Picture.LoadFromFile(FileName);

    newW := Image1.Picture.Width;
    newH := Image1.Picture.Height;

    if oldW <> newW then Image1.Width  := newW;
    if oldH <> newH then Image1.Height := newH;

    if WasBordered then
      Form1.Visible := False;

    if Form1.Width  <> newW then Form1.Width  := newW;
    if Form1.Height <> newH then Form1.Height := newH;

    if Not WasBordered then begin
      if Form1.Left <> (Screen.Width  - Form1.Width)  div 2 then Form1.Left := (Screen.Width  - Form1.Width)  div 2;
      if Form1.Top  <> (Screen.Height - Form1.Height) div 2 then Form1.Top  := (Screen.Height - Form1.Height) div 2;
    end
    else begin
      Form1.Left := centerX - Form1.Width div 2;
      Form1.Top  := centerY - Form1.Height div 2;
    end;

    if WasBordered then
      Form1.Visible := True;
  except
    on E:EInvalidGraphic do; // Image1.Picture := nil; prior image will remain showing
  end;

  Timer1.Enabled := Duration <> 0;
end;


procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key in [#27, 'q', 'Q'] then begin
    Application.Terminate;
    Exit;
  end;

  if Key = #32 then begin
    if Increment < 1 then begin
      Increment := 1;
    end;
    LoadImageFile();
  end
  else
  if Key = #8 then begin
    if Increment > -1 then begin
      Increment := -1;
    end;
    LoadImageFile();
  end
  else
  if Key in ['h', 'H', '?'] then begin
    Timer1.Enabled := False;
    Usage();
    Timer1.Enabled := Duration <> 0;
  end;
end;


procedure TForm1.Image1ContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
  if Form1.BorderStyle = bsNone then begin
    Form1.BorderStyle := bsSingle;
    WasBordered := True;
  end
  else
    Form1.BorderStyle := bsNone;
end;


procedure TForm1.Usage();
begin
        Alert (   'An image viewer by Richard A. DeVenezia'
              +#10'http://www.devenezia.com'
              +#10
              +#10+Caption+' displays bmp, jpg and png files'
              +#10
              +#10'Usage: '
              +#10' '+Caption+' <file> <seconds>'
              +#10' '+Caption+' @<file>'
              +#10' '+Caption+' off'
              +#10' '+Caption+' -? | -help'
              +#10
              +#10'<file> can be wildcarded or a folder name.'
              +#10'@<file> the file should contain a list of filenames, one file per line.'
              +#10
              +#10'<seconds> is duration before showing next image, defaults to 8.'
              +#10'If seconds is >0 the images will stop after the last one.'
              +#10'If seconds is <0 the images will loop.'
              +#10'If seconds is =0 the images stay put.'
              +#10
              +#10'A click or double-click advances to next image.'
              +#10'Esc or Q - quit.'
              +#10'Space - Forward order, advance to next image.'
              +#10'Backspace - Reverse order, advance to next image.'
              +#10'H or ? - show this dialog.'
              +#10
              +#10'A right click toggles the window border so you can'
              +#10' move the image. Right click again to hide the border.'
              +#10
              +#10'Run '+caption+' again while it is running'
              +#10' to change what images are shown.'
              +#10
              +#10'Splash uses PNG Delphi by Gustavo Huffenbacher Daud.'
              );
end;

end.
